function jac=asjac(gscalar,escalar,hyp,mexp,emat,nvar,elcon,DETe,nnodes)
% usage:  jac=asjac(gscalar,escalar,hyp,mexp,emat,nvar,elcon,DETe,nnodes)
%
% Assembles jacobians for multi-dimensional elements
% 
% gscalar - a single scalar constant that multiplies the whole jacobian.  
%           Data specification may be omitted by via an empty matrix ([]).
% escalar - column array of element scalars. Each entry multiplies a specific
%           element matrix.  Data specification may be omitted by via an 
%           empty matrix ([]).
% hyp  -    hypermatrix contraction (element distributed) data.  A column 
%           matrix of nodal data.  May be omitted if no data specified.
% mexp -    mesh measure exponent, a scalar.  Each element matrix is 
%           multiplied by DETe^mexp before assembly, where DETe is 
%           the element determinant.
% emat -    element matrix.  May be a npe by npe master element matrix or a
%           npe by npe by nelements array where emat(:,:,i) is an element
%           matrix for the ith element. (npe is nodes per element, 3 for 
%           2D triangles, 8 for 3D bricks, ect)
% nvar -    nodal variable. diag(nvar) post-multiplies the whole jacobian.
%           May be omitted via an empty matrix.
% elcon -   element connectivity table. Contains one row for each
%           element.  The ith row is an ordered list of node numbers
%           comprising the ith element.
% DETe -    A column array of element determinants, with the ith entry
%           corresponding to the ith element.
% nnodes -  total number of nodes
%
% The assmebled jacobian, jac, is returned in sparse matrix storage.

% find the total number of nodes.
nelements = size(elcon,1);

% find the number of nodes per element
npe = size(elcon,2);

% reserve memory for jac.  npe*npe*nelements is probably several times
% the memory actually be required, but this ensures that additional
% memory will not have to be allocated.
jac = sparse([],[],[],nnodes,nnodes,npe*npe*nelements);

% handle gscalar and escalar data
if isempty(escalar),
  escalar = ones(nelements,1);
end
if isempty(gscalar),
  % do nothing
else
  escalar = escalar * gscalar;
end

% handle mesh measure data
if mexp == -1,
  escalar = escalar ./ DETe;
elseif mexp == 0,
  % do nothing
elseif mexp == 1
  escalar = escalar .* DETe;
else
  escalar = escalar .* (DETe .^ mexp);
end

% handle emat data.  select algorithm based on hyper-matrix vs square
% matrix, one master element matrix for all elements vs one element
% matrix per element
if size(emat,1)==npe & size(emat,3)==1
  % square matrix, a single emat matrix for all elements
  for i=1:nelements, 
    asmat=escalar(i)*emat;
    c = elcon(i,:); % array of nodal connections for the current element
    for j=1:npe
      for k=1:npe
	jac(c(j),c(k))=jac(c(j),c(k)) + asmat(j,k);
      end
    end
  end
elseif size(emat,1)==npe & size(emat,3)==nelements
  % square matrix, one emat matrix for each element
  for i=1:nelements, 
    asmat=escalar(i)*emat(:,:,i);
    c = elcon(i,:); % array of nodal connections for the current element
    for j=1:npe
      for k=1:npe
	jac(c(j),c(k))=jac(c(j),c(k)) + asmat(j,k);
      end
    end
  end
elseif size(emat,1)==npe*npe & size(emat,3)==1
  % hyper-matrix, a single emat matrix for all elements
  %
  % check dimensions of hyp
  if isempty(hyp)
    hyp = ones(nnodes,1);
  elseif prod(size(hyp))==nnodes
    hyp = hyp(:);	% hyp has valid dimensions, convert to column
  else
    error('hyp must have exactly one entry for each node')
  end
  %
  asmat = zeros(npe);			% initialize asmat
  for i=1:nelements, 
    c = elcon(i,:); % array of nodal connections for the current element
    hype = hyp(c); % select element hyp contration data from global data
    % contract emat with hype to form asmat, a square matrix which can be
    % used for assembly
    for r=1:npe
      offset = (r-1)*npe
      asmat(r,:) = hype * emat(1+offset:npe+offset,:);
    end
    asmat=escalar(i)*asmat;
    for j=1:npe
      for k=1:npe
	jac(c(j),c(k))=jac(c(j),c(k)) + asmat(j,k);
      end
    end
  end  
elseif size(emat,1)==npe*npe & size(emat,3)==nelements
  % hyper-matrix, one matrix per element
  %
  % check dimensions of hyp
  if isempty(hyp)
    hyp = ones(nnodes,1);
  elseif prod(size(hyp))==nnodes
    hyp = hyp(:);	% hyp has valid dimensions, convert to column
  else
    error('hyp must have exactly one entry for each node')
  end
  %
  asmat = zeros(npe);			% initialize asmat
  for i=1:nelements, 
    c = elcon(i,:); % array of nodal connections for the current element
    hype = hyp(c); % select element hyp contration data from global data
    % contract emat with hype to form asmat, a square matrix which can be
    % used for assembly
    for r=1:npe
      offset = (r-1)*npe
      asmat(r,:) = hype * emat(1+offset:npe+offset,:,i);
    end
    asmat=escalar(i)*asmat;
    for j=1:npe
      for k=1:npe
	jac(c(j),c(k))=jac(c(j),c(k)) + asmat(j,k);
      end
    end
  end  
else
  error('check dimensions of emat and elcon')
end
%
% handle nvar
if isempty(nvar)
  % do nothing
elseif prod(size(nvar))==nnodes
  % nvar data is present and properly dimensioned, process it
  jac = jac * diag(nvar(:));
else
  error('nvar must be empty or have one entry per node')
end

% make jac as sparse as possible
jac=sparse(jac);