function jac=asjac1D(gscalar,escalar,hyp,mexp,lmat_name,nvar);
%  usage:  asjac1D(gscalar,escalar,hyp,mexp,lmat,nvar)
%
%  Assembles 1d finite element jacobians.  Assumes that each basis used 
%  in the element library matrix is of the same degree.  Returns the
%  assembled global jacobian. 
%
%  gscalar - global scalar data (default unity)
%  escalar - element data (default unity)
%  hyp     - hypermatrix contraction data  (default unity)
%  mexp    - mesh measure exponent, a scalar
%  lmat    - element library matrix
%  nvar    - nodal variable (default unity) diag(nvar)
%    multiplies the square matrix resulting from the application of the 
%    other data. 
%
%  Any of the first three arguments may be omitted by supplying an 
%  empty matrix ([]) in their place.    Ones will be inserted for any
%  legally omitted arguments.  nvar may also be omitted.
%
%  mesh data is passed via X1, a global row array containing the location of
%  all grid points, including element internal grid points
%
%  If the global variable SPARSTORE is set to 'yes' prior to calling asjac1D,
%  asjac1D will return a sparse matrix.  Otherwise, full storage will be
%  used. 

global X1               % a row matrix, nodal coordinate  data
global SPARSTORE	% a flag to indicate if sparse storage is desired
load femlib;

% the eval functionality does not HAVE to be used this way, but it is
%necessary for multi-dimensional problems. Thus, it is applied here to maintain
%consistency in the user generated script.
lmat = eval(lmat_name);
 
nnodes = prod(size(X1)); % number of nodes in the mesh
enodes = size(lmat,2);  % num of columns in lmat is num of nodes per element
nelements = (nnodes-1)/(enodes-1); % number of elements

%  check data for validity.  Generate error message and exit if problems
%  are found

%  check to see that X1 is strictly increasing.
for i=1:nnodes-1
  if X1(i+1)-X1(i) <= 0,
    % X1 is not increasing, generate error
    error('nodal coordinates must increase from left to right')
  end
end

%  check to see if library matrix has reasonable dimensions
if enodes >=2 & enodes <=5
  % reasonable values, do nothing
else
  error('library matrix must have between 2  and 5  columns')
end

% check to see that mesh has proper number of nodes for basis chosen
if mod(nnodes-1,enodes-1)==0
  % valid mesh, do nothing
else
  error('number of nodes inconsistent with degree of basis chosen')
end

% check to see that library matrix has proper number of rows
if size(lmat,2)==size(lmat,1)
  % valid size for square matrix
elseif size(lmat,2)^2==size(lmat,1)
  % proper size for hyper-matrix
else
  error('library (hyper)matrix dimensions inconsistent')
end

% check to see that gscalar is a scalar
if isempty(gscalar)
  % empty matrix is a valid argument for gscalar 
elseif prod(size(gscalar))==1
  % gscalar is indeed scalar
else
  error('the first argument must be a scalar')
end

% check the dimensions of escalar, provide for empty argument
if isempty(escalar)
  escalar = ones(nelements,1);
elseif prod(size(escalar))==nelements
  % good, escalar has one entry for each element
else
  error('the second argument must have exactly one entry per element')
end

% check the dimensions of hyp, provide for empty argument
if isempty(hyp)
  hyp = ones(nnodes,1);
elseif prod(size(hyp))==nnodes
  % good, hyp has one entry for each node
  hyp = hyp(:);				% put hyp into column form
else
  error('the third argument must have exactly one entry per node')
end

% check to make sure that there is a value for the mesh measure exponent
if isempty(mexp)
  error('mesh measure exponent must not be empty')
end

% check to make sure that mexp is a scalar
if prod(size(mexp))==1,
  % good, mexp is a scalar
else
  error('mesh measure exponent must be input as a scalar')
end

% check to be sure that mexp has a reasonable value
if mexp < -4 | mexp > 4
  error('mesh measure exponent must be between -4 and 4')
end

% check the dimensions of nvar
if isempty(nvar)
  % empty is a valid value for nvar
elseif prod(size(nvar))==nnodes
  % good, nvar has one entry for each node
else
  error('the last argument must have exactly one entry per node')
end

%  end of argument checks

%  apply the global scalar to the library matrix, provide for ommited arg
if isempty(gscalar),
  % do nothing
else
  lmat = gscalar * lmat;
end

%  element loops for scalar calculations
%  combine mesh measure data with element scalar data
mleft = 1;   %  initialize node addresses for mesh measure calculation
mright = enodes;
if mexp == 1,
  for i=1:nelements
    escalar(i,1) = escalar(i,1) * (X1(mright) - X1(mleft));
    mright = mright + enodes - 1;
    mleft = mleft + enodes - 1;
  end
elseif mexp == -1,
  for i=1:nelements
    escalar(i,1) = escalar(i,1) / (X1(mright) - X1(mleft));
    mright = mright + enodes - 1;
    mleft = mleft + enodes - 1;
  end
elseif mexp ~= 0,
  for i=1:nelements
    escalar(i,1) = escalar(i,1) * (X1(mright) - X1(mleft))^mexp;
    mright = mright + enodes - 1;
    mleft = mleft + enodes - 1;
  end
end
%  do nothing for the case of mexp == 0  

%  reserve memory for the assembled jacobian. use sparse storage if the 
%  global variable SPARSTORE is set to 'yes'.
if isequal(SPARSTORE,'yes')
  bandwidth = (2*enodes) - 1;
  jac = sparse([],[],[],nnodes,nnodes,nnodes*bandwidth); % sparse storage
else
  jac = zeros(nnodes); % full storage
end

%  element loops for matrix calculations
%  one loop for library hyper-matrices, one for square lib matrices

%  initialize left and right node addresses for current element
mleft = 1;
mright = enodes;
if size(lmat,2)==size(lmat,1); 
  for k=1:nelements, % loop for square library matrices
    eljac = lmat * escalar(k);% element jacobian
    jac(mleft:mright,mleft:mright) = jac(mleft:mright,mleft:mright) + eljac;
    % update left and right node addresses for the next element
    mleft = mleft + enodes - 1;
    mright = mright + enodes - 1;
  end 
else,
  for k=1:nelements, % loop for library hyper-matrices
    ehyp = hyp(mleft:mright);   %  get part of hyp for current element
    for i=1:enodes; % contract hyper-matrix
      for j=1:enodes;
        emat(i,j) = ehyp(:)' * lmat(1+(i-1)*enodes:i*enodes,j) ;
      end;
    end;
  eljac = emat * escalar(k);% element jac
  jac(mleft:mright,mleft:mright) = jac(mleft:mright,mleft:mright) + eljac; 
% update left and right node addresses for the next element
  mleft = mleft + enodes - 1;
  mright = mright + enodes -1;
  end
end

% apply the nvar data if present
if isempty(nvar)
  % do nothing
else
  jac = jac * diag(nvar(:));
end
