function res=asres2D(gscalar,escalar,hyp,mexp,elLibMtrx,nvar)
% usage:  res=asres(gscalar,escalar,hyp,mexp,emat,nvar,elcon,DETe,nnodes)
%
% Assembles residuals for multi-dimensional elements
% 
% gscalar - a single scalar constant that multiplies the whole jacobian.  
%           Data specification may be omitted by via an empty matrix ([]).
% escalar - column array of element scalars. Each entry multiplies a specific
%           element matrix.  Data specification may be omitted by via an 
%           empty matrix ([]).
% hyp  -    hypermatrix contraction (element distributed) data.  A column 
%           matrix of nodal data.  May be omitted if no data specified.
% mexp -    mesh measure exponent, a scalar.  Each element matrix is 
%           multiplied by DETe^mexp before assembly, where DETe is 
%           the element determinant.
% emat -    element matrix.  May be a npe by npe master element matrix or a
%           npe by npe by nelements array where emat(:,:,i) is an element
%           matrix for the ith element. (npe is nodes per element, 3 for 
%           2D triangles, 8 for 3D bricks, ect)
% nvar -    nodal variable. diag(nvar) post-multiplies the whole jacobian.
%           May be omitted via an empty matrix.
% elcon -   element connectivity table. Contains one row for each
%           element.  The ith row is an ordered list of node numbers
%           comprising the ith element.
% DETe -    A column array of element determinants, with the ith entry
%           corresponding to the ith element.
% nnodes -  total number of nodes
%
global X1 X2 elcon

load femlib;

% find the total number of nodes.
nelements = size(elcon,1);
nnodes = prod(size(X1));

%************find DETe, DETei, and ETAe *****************************
% reserve memory for output variables
DETe = zeros(nelements,1);
%DETeI = zeros(nelements,1);
ETAe = zeros(2,2,nelements);
% make sure X1e and X2e (which hold the nodal coordinates for the current
% element) are column arrays
X1e = zeros(4,1);
X2e = zeros(4,1);

% loop over all the elements
for i=1:nelements,
  % find the nodal coordinates for the element
  X1e(:) = X1(elcon(i,:));
  X2e(:) = X2(elcon(i,:));

  % calculate ETAe via eq 6.44
  ETAe(1,1,i) = (X2e(4)-X2e(1)+X2e(3)-X2e(2))/4;
  ETAe(1,2,i) = (X1e(4)-X1e(1)+X1e(3)-X1e(2))/4;
  ETAe(2,1,i) = (X2e(2)-X2e(1)+X2e(3)-X2e(4))/4;
  ETAe(2,2,i) = (X1e(2)-X1e(1)+X1e(3)-X1e(4))/4;

  % calculate DETe
  DETe(i) = det(ETAe(:,:,i));
  if DETe(i) <= 0,
    error(['determinant of element ' num2str(i) ' is negative or zero'])
  end
%  DETeI(i) = 1/DETe(i);
end

%****************Calculate the element matrix ***********************
% Calculates element diffusion matrices for linear quad elements, no
% quadrature, modulo determinant and ke.  Ref: FE123, eq 6.47.
%
%Right now this only calculates the 2D diffusion matrix
% B2IKB will be a cell array.  Each cell of B2IKB will be a matrix, so
% B2IKB is a matrix of matrices.  Matlab uses braces {} to specify
% entries in cell arrays.
%B2IKB=cell(2,2);                        % create a 2 by 2 cell array
%B2IKB{1,1} = B211B;
%B2IKB{2,2} = B222B;
%B2IKB{1,2} = B212B;
%B2IKB{2,1} = B221B;

%Analyze the element matrix entered by the user
mtrx_len = length(elLibMtrx);
if(mtrx_len < 5)
  fprintf('element library matrix syntax is incorrect');
else

dimension = elLibMtrx(1);
if(dimension ~= 'B')
fprintf('dimension is incorrect for this function');
end

noOfBases = elLibMtrx(2);
j = str2num(elLibMtrx(3));
k = str2num(elLibMtrx(4));
basis = elLibMtrx(5);

nelements = size(ETAe,3);
emat = zeros(4,4,nelements);           % reserve memory for DIFFe

eval_mtrx = eval(elLibMtrx);

if j==0 & k==0
for el=1:nelements
  %for i=1:2
        emat(:,:,el) = emat(:,:,el) + eval_mtrx;
 % end
end
elseif j==0 & k~=0
for el=1:nelements
  for i=1:2
        emat(:,:,el) = emat(:,:,el) + ETAe(i,k,el)*eval_mtrx;
  end
end
elseif j~=0 & k==0
for el=1:nelements
  for i=1:2
        emat(:,:,el) = emat(:,:,el) + ETAe(i,j,el)*eval_mtrx;
  end
end
elseif j~=0 & k~=0
for el=1:nelements                      % loop over the elements
  for i=1:2
        emat(:,:,el) = emat(:,:,el) + ETAe(i,j,el)*ETAe(k,j,el)*eval_mtrx;
  end
end
end %end j,k if

end %mtrx_len if-else

%*********************************************************************


% find the number of nodes per element
npe = size(elcon,2);

% reserve memory for res
res = zeros(nnodes,1);

% handle gscalar and escalar data
if isempty(escalar),
  escalar = ones(nelements,1);
end
if isempty(gscalar),
  % do nothing
else
  escalar = escalar * gscalar;
end

% handle mesh measure data
if mexp == -1,
  escalar = escalar ./ DETe;
elseif mexp == 0,
  % do nothing
elseif mexp == 1
  escalar = escalar .* DETe;
else
  escalar = escalar .* (DETe .^ mexp);
end

% check for valid nvar
if prod(size(nvar))==nnodes
  % nvar data is present and properly dimensioned
else
  error('nvar must have one entry per node')
end

% put nvar into a column if it is not already
nvar = nvar(:);

% handle emat data.  select algorithm based on hyper-matrix vs square
% matrix, one master element matrix for all elements vs one element
% matrix per element
if size(emat,1)==npe & size(emat,3)==1
  % square matrix, a single matrix for all elements
  for i=1:nelements, 
    asmat=escalar(i)*emat;
    c = elcon(i,:); % array of nodal connections for the current element
    nvare = nvar(c); % select element nvar from global nvar
    rese = asmat * nvare; % compute the element residual
    % assemble the element residual into the global residual
    for j=1:npe
      res(c(j)) = res(c(j)) + rese(j);
    end
  end
elseif size(emat,1)==npe & size(emat,3)==nelements
  % square matrix, one matrix for each element
  for i=1:nelements, 
    asmat=escalar(i)*emat(:,:,i);
    c = elcon(i,:); % array of nodal connections for the current element
    nvare = nvar(c); % select element nvar from global nvar
    rese = asmat * nvare; % compute the element residual
    % assemble the element residual into the global residual
    for j=1:npe
      res(c(j)) = res(c(j)) + rese(j);
    end
  end
elseif size(emat,1)==npe*npe & size(emat,3)==1
  % hyper-matrix, a single matrix for all elements
  asmat = zeros(npe);			% initialize asmat
  for i=1:nelements, 
    c = elcon(i,:); % array of nodal connections for the current element
    hype = hyp(c); % select element hyp contration data from global data
    % contract emat with hype to form asmat, a square matrix which can be
    % used for assembly
    for r=1:npe
      offset = (r-1)*npe
      asmat(r,:) = hype * emat(1+offset:npe+offset,:);
    end
    nvare = nvar(c); % select element nvar from global nvar
    rese = asmat * nvare; % compute the element residual
    for j=1:npe
      res(c(j)) = res(c(j)) + rese(j);
    end
  end  
elseif size(emat,1)==npe*npe & size(emat,3)==nelements
  % hyper-matrix, one matrix per element
  %
  asmat = zeros(npe);			% initialize asmat
  for i=1:nelements, 
    c = elcon(i,:); % array of nodal connections for the current element
    hype = hyp(c); % select element hyp contration data from global data
    % contract emat with hype to form asmat, a square matrix which can be
    % used for assembly
    for r=1:npe
      offset = (r-1)*npe
      asmat(r,:) = hype * emat(1+offset:npe+offset,:,i);
    end
    nvare = nvar(c); % select element nvar from global nvar
    rese = asmat * nvare; % compute the element residual
    for j=1:npe
      res(c(j)) = res(c(j)) + rese(j);
    end
  end  
else
  error('check dimensions of emat and elcon')
end
