function [jac_diri, b_diri] = do_diri(jac,b,diri_list)
% usage:  [jac_diri, b_diri] = do_diri(jac,b,diri_list)
% Applies Dirichlet boundry conditions specified by diri_list to jac and
% b, producing jac_diri and b_diri.  jac and b are not altered.
% diri_list has one row for each node, just as jac and b do.  A one (or
% other non-zero value) in the first column of diri_list means that a
% Dirichlet condition should be applied for the corresponding node.  The
% second column contains the Dirichlet values to be applied.  A Dirichlet 
% condition is applied to the ith node by zeroing out the ith row of
% jac_diri and placing a one in the (i,i) position of jac_diri.  The
% Dirichlet value is written to the ith position of ith position of
% b_diri.

% check to be sure arguments have consistent dimensions
nnodes = size(jac,1);
if size(jac,2) == nnodes;
  % good, jac is square
else
  error('jac is not a square matrix')
end
if size(b,1) == nnodes & prod(size(b)) == nnodes;
  % good, b is dimensioned properly.
else
  error('b must be a column matrix with one entry for each row of jac')
end
if size(diri_list,1) == nnodes 2 & size(diri_list,2) == 2;
  % good, diri_list is properly dimensioned
else
  error('diri_list must be a n nodes by 2 array')
end

% initialize jac_diri and b_diri
% Matlab stores in columns, so transpose the matrix and operate on
% columns.  This is more efficient than operating on rows of the
% untransposed matrix.
jac_diri = jac';
b_diri = b;

col_of_zeros = spalloc(nnodes,1,0);

% loop over nodes, and apply Dirichlet conditions
for i=1:nnodes
  if diri_list(i,1) == 0
    % this col is not diri, skip it.
  else
    jac_diri(:,i) = col_of_zeros;
    jac_diri(i,i) = 1;
    b_diri(i) = diri_list(i,2);
  end
end

% transpose back
jac_diri = jac_diri';